package net.maunium.Maucros.ASM.transformers;

import java.util.Iterator;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.InsnNode;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.MethodInsnNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.tree.VarInsnNode;

import net.maunium.Maucros.ASM.AbstractMauTransformer;
import net.maunium.Maucros.Actions.ActionFly;
import net.maunium.Maucros.Actions.ActionNofall;
import net.maunium.Maucros.Logging.ChatLogger;

public class EntityPlayerSPT extends AbstractMauTransformer {
	public EntityPlayerSPT() {
		super("net.minecraft.client.entity.EntityPlayerSP", "cio");
	}
	
	@Override
	public byte[] transform(byte[] bytes, boolean obf) {
		long st = System.currentTimeMillis();
		String method1 = obf ? "m" : "onLivingUpdate";
		String method2 = obf ? "a" : "addChatMessage";
		String desc2 = obf ? "(Lfj;)V" : "(Lnet/minecraft/util/IChatComponent;)V";
		log.info("Going to patch " + method1 + "() and " + method2 + "(IChatComponent msg) in class " + getName(obf) + "...");
		
		log.trace("Creating ClassNode");
		ClassNode node = new ClassNode();
		log.trace("Creating ClassReader");
		ClassReader cr = new ClassReader(bytes);
		log.trace("Accepting node to reader");
		cr.accept(node, 0);
		
		log.debug("Looping through methods...");
		int done = 0;
		for (MethodNode m : node.methods){
			if (m.name.equals(method1) && m.desc.equals("()V")) {
				log.info("Found method " + m.name + " with desc " + m.desc + ".");
				AbstractInsnNode current;
				
				Iterator<AbstractInsnNode> iter = m.instructions.iterator();
				int spec = 0;
				int pif = 0;
				int success = 0;
				LabelNode prev1 = null, prev2 = null;
				log.debug("Iterating through bytecode instructions.");
				while (iter.hasNext()) {
					current = iter.next();
					if (current.getOpcode() == Opcodes.PUTFIELD && current.getType() == InsnNode.FIELD_INSN) {
						if (pif == 21) {
							log.debug("Found 22th PUTFIELD with type FIELD_INSN.");
							log.trace("Adding MethodInsnNode to call ActionNofall.exec() for nofall functionality");
							m.instructions.insert(current, new MethodInsnNode(Opcodes.INVOKESTATIC, ActionNofall.class.getName().replace(".", "/"), "exec", "()V", false));
							success++;
						}
						pif++;
					} else if (current.getOpcode() == Opcodes.INVOKESPECIAL) {
						if (spec == 1) {
							log.debug("Found 2nd INVOKESPECIAL");
							InsnList inject = new InsnList();
							log.trace("Adding MethodInsnNode to call ActionNofall.exec() for nofall functionality");
							inject.add(new MethodInsnNode(Opcodes.INVOKESTATIC, ActionNofall.class.getName().replace(".", "/"), "exec", "()V", false));
							log.trace("Adding MethodInsnNode to call ActionFly.exec() for fly functionality");
							inject.add(new MethodInsnNode(Opcodes.INVOKESTATIC, ActionFly.class.getName().replace('.', '/'), "exec", "()V", false));
							m.instructions.insert(current, inject);
							success++;
						}
						spec++;
					} else if (current.getOpcode() == -1 && current instanceof LabelNode) {
						prev2 = prev1;
						prev1 = (LabelNode) current;
					}
				}
				log.debug("Finished patching " + method1 + "()! Success: " + (success == 2));
				done++;
			} else if (m.name.equals(method2) && m.desc.equals(desc2)) {
				log.info("Found method " + m.name + " with desc " + m.desc + ".");
				InsnList inject = new InsnList();
				
				inject.add(new VarInsnNode(Opcodes.ALOAD, 1));
				inject.add(new MethodInsnNode(Opcodes.INVOKESTATIC, ChatLogger.class.getName().replace('.', '/'), "printChat",
						"(Lnet/minecraft/util/IChatComponent;)V", false));
				inject.add(new InsnNode(Opcodes.RETURN));
				m.instructions.insert(inject);
				done++;
			}
			if (done == 2) break;
		}
		log.debug("Writing changes...");
		ClassWriter cw = new ClassWriter(ClassWriter.COMPUTE_MAXS | ClassWriter.COMPUTE_FRAMES);
		node.accept(cw);
		long tt = System.currentTimeMillis() - st;
		log.info("Patching complete in " + tt + "ms. Returning patched class.");
		return cw.toByteArray();
	}
}
