package net.maunium.Maucros.ASM.transformers;

import java.util.Iterator;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.FieldInsnNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.InsnNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.tree.VarInsnNode;

import net.maunium.Maucros.ASM.AbstractMauTransformer;
import net.maunium.Maucros.Config.Settings;

public class EntityPlayerT extends AbstractMauTransformer {
	
	public EntityPlayerT() {
		super("net.minecraft.entity.player.EntityPlayer", "ahd");
	}
	
	@Override
	public byte[] transform(byte[] bytes, boolean obf) {
		long st = System.currentTimeMillis();
		String method = obf ? "aR" : "getEyeHeight";
		log.info("Going to patch " + method + "() in class " + getName(obf) + "...");
		
		log.trace("Creating ClassNode");
		ClassNode node = new ClassNode();
		log.trace("Creating ClassReader");
		ClassReader cr = new ClassReader(bytes);
		log.trace("Accepting node to reader");
		cr.accept(node, 0);
		
		log.debug("Looping through methods...");
		int done = 0;
		for (MethodNode m : node.methods) {
			if (m.name.equals(method) && m.desc.equals("()F")) {
				log.info("Found method " + m.name + " with desc " + m.desc + ".");
				AbstractInsnNode current = null;
				
				Iterator<AbstractInsnNode> iter = m.instructions.iterator();
				Instructions: while (iter.hasNext()) {
					current = iter.next();
					if (current.getOpcode() == Opcodes.FSUB) break Instructions;
				}
				
				if (current == null) {
					log.fatal("Couldn't find injection point");
					return bytes;
				}
				
				InsnList inject = new InsnList();
				
				LabelNode l = new LabelNode();
				inject.add(new FieldInsnNode(Opcodes.GETSTATIC, Settings.Enabled.class.getName().replace('.', '/'), "fly", "I"));
				inject.add(new InsnNode(Opcodes.ICONST_1));
				inject.add(new JumpInsnNode(Opcodes.IF_ICMPNE, l));
				inject.add(new VarInsnNode(Opcodes.FLOAD, 1));
				inject.add(new InsnNode(Opcodes.FRETURN));
				inject.add(l);
				
				m.instructions.insert(current, inject);
			}
		}
		log.debug("Writing changes...");
		ClassWriter cw = new ClassWriter(ClassWriter.COMPUTE_MAXS | ClassWriter.COMPUTE_FRAMES);
		node.accept(cw);
		long tt = System.currentTimeMillis() - st;
		log.info("Patching complete in " + tt + "ms. Returning patched class.");
		return cw.toByteArray();
	}
	
}
