package net.maunium.Maucros.ASM.transformers;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.FieldInsnNode;
import org.objectweb.asm.tree.FrameNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.InsnNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.MethodInsnNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.tree.VarInsnNode;

import net.maunium.Maucros.ASM.AbstractMauTransformer;
import net.maunium.Maucros.Actions.ActionBlink;
import net.maunium.Maucros.Config.Settings;

public class NetworkManagerT extends AbstractMauTransformer {
	
	public NetworkManagerT() {
		super("net.minecraft.network.NetworkManager", "gr");
	}
	
	@Override
	public byte[] transform(byte[] bytes, boolean obf) {
		long st = System.currentTimeMillis();
		String methodName = obf ? "a" : "dispatchPacket";
		String packet = obf ? "id" : "net/minecraft/network/Packet";
		String desc = "(L" + packet + ";[Lio/netty/util/concurrent/GenericFutureListener;)V";
		log.info("Going to patch " + methodName + "(" + (obf ? "ft" : "Packet") + " p) in class " + getName(obf) + "...");
		
		log.trace("Creating ClassNode");
		ClassNode node = new ClassNode();
		log.trace("Creating ClassReader");
		ClassReader cr = new ClassReader(bytes);
		log.trace("Accepting node to reader");
		cr.accept(node, 0);
		
		for (MethodNode m : node.methods) {
			if (m.name.equals(methodName) && m.desc.equals(desc)) {
				log.info("Found method " + m.name + " with desc " + m.desc + ".");
				AbstractInsnNode current;
				
				InsnList inject = new InsnList();
				log.trace("Adding Frame Node (F_SAME)");
				inject.add(new FrameNode(Opcodes.F_SAME, 0, null, 0, null));
				log.trace("Adding Field Node (GETSTATIC)");
				inject.add(new FieldInsnNode(Opcodes.GETSTATIC, Settings.Enabled.class.getName().replace('.', '/'), "blink", "Z"));
				log.trace("Adding Jump Node (IFEQ, Label 1)");
				LabelNode ln = new LabelNode();
				inject.add(new JumpInsnNode(Opcodes.IFEQ, ln));
				log.trace("Adding Variable Node (ALOAD, 1)");
				inject.add(new VarInsnNode(Opcodes.ALOAD, 1));
				log.trace("Adding Method Node (INVOKESTATIC, ActionBlink.blink(Packet p))");
				inject.add(new MethodInsnNode(Opcodes.INVOKESTATIC, ActionBlink.class.getName().replace(".", "/"), "blink", "(Lnet/minecraft/network/Packet;)V", false));
				log.trace("Adding Node (Return)");
				inject.add(new InsnNode(Opcodes.RETURN));
				log.trace("Adding Label 1");
				inject.add(ln);
				log.trace("Inserting node list to start of method instructuons");
				m.instructions.insert(inject);
			}
		}
		
		log.debug("Writing changes...");
		ClassWriter cw = new ClassWriter(ClassWriter.COMPUTE_MAXS | ClassWriter.COMPUTE_FRAMES);
		node.accept(cw);
		long tt = System.currentTimeMillis() - st;
		log.info("Patching complete in " + tt + "ms. Returning patched class.");
		return cw.toByteArray();
	}
}
