package net.maunium.Maucros.Actions;

import java.util.List;

import net.maunium.Maucros.Misc.MaucrosLicensor;

import net.minecraft.util.EnumChatFormatting;

/**
 * The base class for all Maucros Actions.
 * 
 * @author Tulir293
 * @since 1.4 Alpha 7
 */
public abstract class Action {
	/**
	 * The name of the action.
	 */
	private final String name;
	
	/**
	 * Creates a new action.
	 * 
	 * @param name The name of the action.
	 */
	public Action(String name) {
		if (!MaucrosLicensor.check()) throw new RuntimeException("Maucros not licenced");
		this.name = name;
	}
	
	/**
	 * @return The name of the action.
	 */
	public final String getName() {
		return name;
	}
	
	/**
	 * The text to render if this action is on. Doesn't yet support non-official actions.
	 * 
	 * @param str The list to add the render texts to.
	 */
	public void renderText(List<String> str) {
		str.add(name + EnumChatFormatting.GREEN + " ON");
	}
	
	/**
	 * Executes this action. This is usually called every tick.
	 */
	public abstract void execute();
}
