package net.maunium.Maucros.Actions;

import net.maunium.Maucros.Config.Settings;

import net.minecraft.client.Minecraft;

/**
 * A version of Action that executes the action at specific intervals instead of every time execute is called.
 * 
 * @author Tulir293
 * @since 1.4 Alpha 7
 */
public abstract class DelayAction extends Action {
	/**
	 * The delay to wait before executing the action again after executing once.
	 */
	private long delay;
	/**
	 * The time of the previous execution from {@link Minecraft#getSystemTime()}
	 */
	private long lastExecution = 0;
	
	/**
	 * Creates a new delay action.
	 * 
	 * @param name The name of the action.
	 */
	public DelayAction(String name) {
		super(name);
		this.delay = refreshDelay();
	}
	
	/**
	 * @return The {@link #delay} to wait before executing the action again after executing once.
	 */
	public final long getDelay() {
		return delay;
	}
	
	/**
	 * Called when the delay has passed and the action should be executed again.
	 */
	public abstract void executeChecked();
	
	/**
	 * Called when the delay value needs to be updated.
	 */
	public abstract int refreshDelay();
	
	@Override
	public final void execute() {
		if (Settings.disabled) return;
		if (Minecraft.getSystemTime() - lastExecution > delay) {
			executeChecked();
			this.delay = refreshDelay();
			lastExecution = Minecraft.getSystemTime();
		}
	}
	
}
