package net.maunium.Maucros.Config;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import net.maunium.Maucros.Maucros;
import net.maunium.Maucros.KeyMaucros.KeyMaucro;
import net.maunium.Maucros.KeyMaucros.LuaKeyMaucro;
import net.maunium.Maucros.Logging.MaucrosLogger;
import net.maunium.Maucros.Misc.MauFileUtils;

import net.minecraft.client.Minecraft;
import net.minecraft.entity.EntityLivingBase;

/**
 * Use the subclasses to load/save configurations.
 * 
 * @author Tulir293
 * @since Unknown developement version in post-1.0 stage
 */
public class Configurations {
	private static File workDir;
	
	/**
	 * Get the working directory.
	 */
	public static File getWorkDir() {
		if (workDir == null) workDir = new File(Minecraft.getMinecraft().mcDataDir + File.separator + "config" + File.separator + "Maucros");
		if (!workDir.exists()) workDir.mkdirs();
		return workDir;
	}
	
	/**
	 * Get a directory inside the working directory.
	 */
	public static File getDirectory(String directory) {
		File f = new File(getWorkDir() + File.separator + directory);
		if (!f.exists()) f.mkdir();
		return f;
	}
	
	/**
	 * Get a file inside a directory inside the working directory.
	 */
	public static File getFile(String directory, String fileName) {
		File f = new File(Configurations.getDirectory(directory), fileName + ".maudat");
		try {
			if (!f.exists()) f.createNewFile();
		} catch (IOException e) {
			Maucros.getLogger().catching(e);
		}
		return f;
	}
	
	/**
	 * Get a file inside a directory inside the working directory, but don't create it if it doesn't exist.
	 */
	public static File getFileNoMk(String directory, String fileName) {
		File f = new File(Configurations.getDirectory(directory), fileName + ".maudat");
		return f;
	}
	
	/**
	 * Get a file inside the working directory.
	 */
	public static File getFile(String fileName) {
		File f = new File(Configurations.getWorkDir(), fileName + ".maudat");
		try {
			if (!f.exists()) f.createNewFile();
		} catch (IOException e) {
			Maucros.getLogger().catching(e);
		}
		return f;
	}
	
	/**
	 * Get a file inside the working directory, but don't create it if it doesn't exist.
	 */
	public static File getFileNoMk(String fileName) {
		File f = new File(Configurations.getWorkDir(), fileName + ".maudat");
		return f;
	}
	
	public static class Freecam {
		private static Properties props;
		
		public static void save() throws IOException {
			File f = Configurations.getFile("freecam");
			if (props == null) props = new Properties();
			
			props.setProperty("move.forward", "" + Settings.Freecam.Move.forward);
			props.setProperty("move.back", "" + Settings.Freecam.Move.back);
			props.setProperty("move.right", "" + Settings.Freecam.Move.right);
			props.setProperty("move.left", "" + Settings.Freecam.Move.left);
			props.setProperty("move.up", "" + Settings.Freecam.Move.up);
			props.setProperty("move.down", "" + Settings.Freecam.Move.down);
			
			props.setProperty("view.downleft", "" + Settings.Freecam.View.downleft);
			props.setProperty("view.down", "" + Settings.Freecam.View.down);
			props.setProperty("view.downright", "" + Settings.Freecam.View.downright);
			props.setProperty("view.left", "" + Settings.Freecam.View.left);
			props.setProperty("view.reset", "" + Settings.Freecam.View.reset);
			props.setProperty("view.right", "" + Settings.Freecam.View.right);
			props.setProperty("view.upleft", "" + Settings.Freecam.View.upleft);
			props.setProperty("view.up", "" + Settings.Freecam.View.up);
			props.setProperty("view.upright", "" + Settings.Freecam.View.upright);
			props.setProperty("view.usemouse", "" + Settings.Freecam.View.useMouse);
			
			props.store(new FileOutputStream(Configurations.getFile("config")), null);
		}
		
		public static void load() throws IOException {
			File f = Configurations.getFile("freecam");
			props = new Properties();
			if (Configurations.getFileNoMk("config").exists()) {
				props.load(new FileInputStream(Configurations.getFile("config")));
				if (props.containsKey("move.forward")) Settings.Freecam.Move.forward = Integer.parseInt(props.get("move.forward").toString());
				if (props.containsKey("move.back")) Settings.Freecam.Move.back = Integer.parseInt(props.get("move.back").toString());
				if (props.containsKey("move.right")) Settings.Freecam.Move.right = Integer.parseInt(props.get("move.right").toString());
				if (props.containsKey("move.left")) Settings.Freecam.Move.left = Integer.parseInt(props.get("move.left").toString());
				if (props.containsKey("move.up")) Settings.Freecam.Move.up = Integer.parseInt(props.get("move.up").toString());
				if (props.containsKey("move.down")) Settings.Freecam.Move.down = Integer.parseInt(props.get("move.down").toString());
				
				if (props.containsKey("view.downleft")) Settings.Freecam.View.downleft = Integer.parseInt(props.get("view.downleft").toString());
				if (props.containsKey("view.down")) Settings.Freecam.View.down = Integer.parseInt(props.get("view.down").toString());
				if (props.containsKey("view.downright")) Settings.Freecam.View.downright = Integer.parseInt(props.get("view.downright").toString());
				if (props.containsKey("view.left")) Settings.Freecam.View.left = Integer.parseInt(props.get("view.left").toString());
				if (props.containsKey("view.reset")) Settings.Freecam.View.reset = Integer.parseInt(props.get("view.reset").toString());
				if (props.containsKey("view.right")) Settings.Freecam.View.right = Integer.parseInt(props.get("view.right").toString());
				if (props.containsKey("view.upleft")) Settings.Freecam.View.upleft = Integer.parseInt(props.get("view.upleft").toString());
				if (props.containsKey("view.up")) Settings.Freecam.View.up = Integer.parseInt(props.get("view.up").toString());
				if (props.containsKey("view.upright")) Settings.Freecam.View.upright = Integer.parseInt(props.get("view.upright").toString());
				if (props.containsKey("view.usemouse")) Settings.Freecam.View.useMouse = Boolean.parseBoolean(props.get("view.usemouse").toString());
			}
		}
	}
	
	public static class XRay {
		public static void save() throws IOException {
			File f = Configurations.getFile("xray-blocks");
			MauFileUtils.clear(f);
			MauFileUtils.write(f, Settings.XRay.getDisabledBlocks());
		}
		
		public static void load() throws IOException {
			File f = Configurations.getFile("xray-blocks");
			Set<Integer> xrays = new HashSet<Integer>();
			for(String s : MauFileUtils.read(f)){
				try{
					xrays.add(Integer.parseInt(s));
				} catch (NumberFormatException e){}
			}
			Settings.XRay.init(xrays);
		}
	}
	
	public static class Alts {
		public static void save() throws IOException {
			File f = Configurations.getFile("alts");
			MauFileUtils.clear(f);
			
			List<String> alts = new ArrayList<String>();
			for (String s : Settings.Alts.getAlts())
				alts.add(s + ":" + Settings.Alts.getPassword(s));
			MauFileUtils.write(f, alts);
		}
		
		public static void load() throws IOException {
			for (String s : MauFileUtils.read(Configurations.getFile("alts")))
				if (s.contains(":")) {
					String[] t = s.split(":", 2);
					Settings.Alts.addAlt(t[0], t[1]);
				}
		}
	}
	
	public static class Generic {
		private static Properties props;
		
		public static void save() throws IOException {
			if (props == null) props = new Properties();
			
			props.setProperty("attacking.range", Settings.Attacking.range + "");
			
			props.setProperty("autosoup.delay", Settings.Autosoup.delay + "");
			props.setProperty("autosoup.makeseemlegit", Settings.Autosoup.makeseemlegit + "");
			
			props.setProperty("fly.speed", Settings.Fly.getSpeed() + "");
			props.setProperty("fly.jumpheight", Settings.Fly.jumpHeight + "");
			props.setProperty("fly.maxspeed", Settings.Fly.getMaxSpeed() + "");
			props.setProperty("fly.minspeed", Settings.Fly.getMinSpeed() + "");
			
			props.setProperty("blink.safety", Settings.Blink.safetyLevel + "");
			props.setProperty("blink.automated", Settings.Blink.automated + "");
			
			props.setProperty("screenshot.ip", Settings.Screenshot.ip);
			props.setProperty("screenshot.port", Settings.Screenshot.port + "");
			props.setProperty("screenshot.password", Settings.Screenshot.password);
			
			props.setProperty("phase.automated", Settings.Phase.automated + "");
			props.setProperty("phase.autoforward", Settings.Phase.autoforward + "");
			
			props.setProperty("autouse.delay", Settings.Delay.autouse + "");
			props.setProperty("autoattack.delay", Settings.Autoattack.delay + "");
			props.setProperty("spammer.delay", Settings.Delay.spammer + "");
			props.setProperty("attackaura.delay", Settings.Delay.attackaura + "");
			props.setProperty("aimbot.delay", Settings.Delay.aimbot + "");
			
			props.setProperty("encryption.password", Settings.Encryption.password);
			props.setProperty("encryption.salt", Settings.Encryption.salt);
			props.setProperty("encryption.base64", Settings.Encryption.base64 + "");
			
			props.setProperty("dev.debugtype", Maucros.getLogger().getDebugType() + "");
			props.setProperty("dev.logtype", Maucros.getLogger().getLogType() + "");
			props.setProperty("dev.autoflushlog", MaucrosLogger.flushLineCount + "");
			if (Settings.language != null) props.setProperty("language", Settings.language);
			else if (props.containsKey("language")) props.remove("language");
			
			props.store(new FileOutputStream(Configurations.getFile("config")), null);
		}
		
		public static void load() throws IOException {
			if (!LuaKeyMaucro.f.exists()) LuaKeyMaucro.f.mkdirs();
			props = new Properties();
			if (Configurations.getFileNoMk("config").exists()) {
				props.load(new FileInputStream(Configurations.getFile("config")));
				if (props.containsKey("attacking.range")) Settings.Attacking.range = Integer.parseInt(props.getProperty("attacking.range"));
				
				if (props.containsKey("autosoup.delay")) Settings.Autosoup.delay = Integer.parseInt(props.getProperty("autosoup.delay"));
				if (props.containsKey("autosoup.makeseemlegit"))
					Settings.Autosoup.makeseemlegit = Boolean.parseBoolean(props.getProperty("autosoup.makeseemlegit"));
				
				if (props.containsKey("fly.speed")) Settings.Fly.setSpeed(Integer.parseInt(props.getProperty("fly.speed")));
				if (props.containsKey("fly.jumpheight")) Settings.Fly.jumpHeight = Double.parseDouble(props.getProperty("fly.jumpheight"));
				if (props.containsKey("fly.maxspeed")) Settings.Fly.setMaxSpeed(Integer.parseInt(props.getProperty("fly.maxspeed")));
				if (props.containsKey("fly.minspeed")) Settings.Fly.setMinSpeed(Integer.parseInt(props.getProperty("fly.minspeed")));
				
				if (props.containsKey("blink.safety")) Settings.Blink.safetyLevel = Integer.parseInt(props.getProperty("blink.safety"));
				if (props.containsKey("blink.automated")) Settings.Blink.automated = Boolean.parseBoolean(props.getProperty("blink.automated"));
				
				if (props.containsKey("screenshot.ip")) Settings.Screenshot.ip = props.getProperty("screenshot.ip");
				if (props.containsKey("screenshot.port")) Settings.Screenshot.port = Integer.parseInt(props.getProperty("screenshot.port"));
				if (props.containsKey("screenshot.password")) Settings.Screenshot.password = props.getProperty("screenshot.password");
				
				if (props.containsKey("phase.automated")) Settings.Phase.automated = Boolean.parseBoolean(props.getProperty("phase.automated"));
				if (props.containsKey("phase.autoforward")) Settings.Phase.autoforward = Boolean.parseBoolean(props.getProperty("phase.autoforward"));
				
				if (props.containsKey("autouse.delay")) Settings.Delay.autouse = Integer.parseInt(props.getProperty("autouse.delay"));
				if (props.containsKey("autoattack.delay")) Settings.Autoattack.delay = Integer.parseInt(props.getProperty("autoattack.delay"));
				if (props.containsKey("spammer.delay")) Settings.Delay.spammer = Integer.parseInt(props.getProperty("spammer.delay"));
				if (props.containsKey("attackaura.delay")) Settings.Delay.attackaura = Integer.parseInt(props.getProperty("attackaura.delay"));
				if (props.containsKey("aimbot.delay")) Settings.Delay.aimbot = Integer.parseInt(props.getProperty("aimbot.delay"));
				
				if (props.containsKey("encryption.password")) Settings.Encryption.password = props.getProperty("encryption.password");
				if (props.containsKey("encryption.salt")) Settings.Encryption.salt = props.getProperty("encryption.salt");
				if (props.containsKey("encryption.base64")) Settings.Encryption.base64 = Boolean.parseBoolean(props.getProperty("encryption.base64"));
				
				if (props.containsKey("dev.debugtype")) Maucros.getLogger().setDebugType(Integer.parseInt(props.getProperty("dev.debugtype")));
				if (props.containsKey("dev.logtype")) Maucros.getLogger().setLogType(Integer.parseInt(props.getProperty("dev.logtype")));
				if (props.containsKey("dev.autoflushlog")) MaucrosLogger.flushLineCount = Integer.parseInt(props.getProperty("dev.autoflushlog"));
				if (props.containsKey("language")) Settings.language = props.getProperty("language");
				
			}
		}
	}
	
	public static class Friends {
		public static void save() throws IOException {
			File f = Configurations.getFile("friends");
			MauFileUtils.clear(f);
			MauFileUtils.write(f, Settings.Friends.getFriends());
		}
		
		public static void load() throws IOException {
			for (String s : MauFileUtils.read(Configurations.getFile("friends")))
				if (!s.isEmpty() && !s.equals("_")) Settings.Friends.addFriend(s);
		}
	}
	
	public static class Targets {
		public static void save() throws IOException {
			File f = Configurations.getFile("targets");
			MauFileUtils.clear(f);
			MauFileUtils.write(f, Settings.Attacking.getTargets());
		}
		
		public static void load() throws IOException {
			for (String s : MauFileUtils.read(Configurations.getFile("targets"))) {
				if (s.isEmpty()) continue;
				Class<? extends EntityLivingBase> c;
				try {
					c = (Class<? extends EntityLivingBase>) Class.forName(s.substring(6));
				} catch (Throwable t) {
					continue;
				}
				if (c != null) Settings.Attacking.addTarget(c);
			}
		}
	}
	
	public static class KeyMaucros {
		public static void save() throws IOException {
			MauFileUtils.write(Configurations.getFile("keymaucros"), Settings.KeyMaucros.getKeyMaucros());
		}
		
		public static void load() throws IOException {
			List<String> conf = MauFileUtils.read(Configurations.getFile("keymaucros"));
			if (conf == null) {
				Settings.KeyMaucros.init(new ArrayList<KeyMaucro>());
				return;
			} else if (conf.size() <= 0) {
				Settings.KeyMaucros.init(new ArrayList<KeyMaucro>());
				return;
			}
			
			List<KeyMaucro> kms = new ArrayList<KeyMaucro>();
			for (String s : conf)
				try {
					kms.add(KeyMaucro.parseKeyMaucro(s));
				} catch (Exception e) {
					Maucros.getLogger().catching(e);
					continue;
				}
			Settings.KeyMaucros.init(kms);
		}
	}
}