package net.maunium.Maucros.Gui.Config;

import java.io.IOException;

import com.mcf.davidee.guilib.basic.Label;
import com.mcf.davidee.guilib.core.Button;
import com.mcf.davidee.guilib.core.Container;
import com.mcf.davidee.guilib.vanilla.ButtonVanilla;
import com.mcf.davidee.guilib.vanilla.ScrollbarVanilla;

import net.maunium.Maucros.Maucros;
import net.maunium.Maucros.Config.Configurations;
import net.maunium.Maucros.Config.Settings;
import net.maunium.Maucros.Gui.MauScreen;
import net.maunium.Maucros.Gui.Widgets.FocusableButton;
import net.maunium.Maucros.Gui.Widgets.KeySelectButton;
import net.maunium.Maucros.Gui.Widgets.ShiftableLabel;
import net.maunium.Maucros.Gui.Widgets.StateButton;
import net.maunium.Maucros.Misc.I18n;

public class GuiFreecamKeys extends MauScreen {
	public GuiFreecamKeys(GuiConfig parent) {
		super(parent);
	}
	
	private Container c, c2;
	private ScrollbarVanilla sb;
	private Label title;
	private ShiftableLabel moveTitle, viewTitle;
	private KeySelectButton[] move = new KeySelectButton[6];
	private ShiftableLabel[] moveLabel = new ShiftableLabel[6];
	private KeySelectButton[] view = new KeySelectButton[9];
	private ShiftableLabel[] viewLabel = new ShiftableLabel[9];
	private StateButton viewMouse, moveDefault;
	private ButtonVanilla back;
	
	@Override
	public void onInit() {
		title = new Label(I18n.format("conf.freecam.title"));
		back = new ButtonVanilla(I18n.format("conf.back"), this);
		moveTitle = new ShiftableLabel(I18n.format("conf.freecam.move.title"));
		viewTitle = new ShiftableLabel(I18n.format("conf.freecam.view.title"));
		
		viewMouse = new StateButton(150, 20, I18n.format("conf.freecam.view.usemouse"), this, Settings.Freecam.View.useMouse ? 1 : 0);
		moveDefault = new StateButton(150, 20, I18n.format("conf.freecam.move.default"), this, Settings.Freecam.Move.useDefault ? 1 : 0);
		
		move[0] = new KeySelectButton(this, Settings.Freecam.Move.forward);
		move[1] = new KeySelectButton(this, Settings.Freecam.Move.back);
		move[2] = new KeySelectButton(this, Settings.Freecam.Move.right);
		move[3] = new KeySelectButton(this, Settings.Freecam.Move.left);
		move[4] = new KeySelectButton(this, Settings.Freecam.Move.up);
		move[5] = new KeySelectButton(this, Settings.Freecam.Move.down);
		
		moveLabel[0] = new ShiftableLabel(I18n.format("conf.freecam.move.forward"), false);
		moveLabel[1] = new ShiftableLabel(I18n.format("conf.freecam.move.back"), false);
		moveLabel[2] = new ShiftableLabel(I18n.format("conf.freecam.move.right"), false);
		moveLabel[3] = new ShiftableLabel(I18n.format("conf.freecam.move.left"), false);
		moveLabel[4] = new ShiftableLabel(I18n.format("conf.freecam.move.up"), false);
		moveLabel[5] = new ShiftableLabel(I18n.format("conf.freecam.move.down"), false);
		
		view[0] = new KeySelectButton(this, Settings.Freecam.View.downleft);
		view[1] = new KeySelectButton(this, Settings.Freecam.View.down);
		view[2] = new KeySelectButton(this, Settings.Freecam.View.downright);
		view[3] = new KeySelectButton(this, Settings.Freecam.View.left);
		view[4] = new KeySelectButton(this, Settings.Freecam.View.reset);
		view[5] = new KeySelectButton(this, Settings.Freecam.View.right);
		view[6] = new KeySelectButton(this, Settings.Freecam.View.upleft);
		view[7] = new KeySelectButton(this, Settings.Freecam.View.up);
		view[8] = new KeySelectButton(this, Settings.Freecam.View.upright);
		
		viewLabel[0] = new ShiftableLabel(I18n.format("conf.freecam.view.downleft"), false);
		viewLabel[1] = new ShiftableLabel(I18n.format("conf.freecam.view.down"), false);
		viewLabel[2] = new ShiftableLabel(I18n.format("conf.freecam.view.downright"), false);
		viewLabel[3] = new ShiftableLabel(I18n.format("conf.freecam.view.left"), false);
		viewLabel[4] = new ShiftableLabel(I18n.format("conf.freecam.view.reset"), false);
		viewLabel[5] = new ShiftableLabel(I18n.format("conf.freecam.view.right"), false);
		viewLabel[6] = new ShiftableLabel(I18n.format("conf.freecam.view.upleft"), false);
		viewLabel[7] = new ShiftableLabel(I18n.format("conf.freecam.view.up"), false);
		viewLabel[8] = new ShiftableLabel(I18n.format("conf.freecam.view.upright"), false);
		
		sb = new ScrollbarVanilla(10);
		c2 = new Container(sb, 14, 12);
		c = new Container();
		
		c2.addWidgets(move);
		c2.addWidgets(moveLabel);
		c2.addWidgets(view);
		c2.addWidgets(viewLabel);
		c2.addWidgets(viewMouse, moveDefault, moveTitle, viewTitle);
		c.addWidgets(title, back);
		
		containers.add(c);
		containers.add(c2);
	}
	
	@Override
	public void save() {
		Settings.Freecam.Move.forward = move[0].getKeycode();
		Settings.Freecam.Move.back = move[1].getKeycode();
		Settings.Freecam.Move.right = move[2].getKeycode();
		Settings.Freecam.Move.left = move[3].getKeycode();
		Settings.Freecam.Move.up = move[4].getKeycode();
		Settings.Freecam.Move.down = move[5].getKeycode();
		
		Settings.Freecam.View.downleft = view[0].getKeycode();
		Settings.Freecam.View.down = view[1].getKeycode();
		Settings.Freecam.View.downright = view[2].getKeycode();
		Settings.Freecam.View.left = view[3].getKeycode();
		Settings.Freecam.View.reset = view[4].getKeycode();
		Settings.Freecam.View.right = view[5].getKeycode();
		Settings.Freecam.View.upleft = view[6].getKeycode();
		Settings.Freecam.View.up = view[7].getKeycode();
		Settings.Freecam.View.upright = view[8].getKeycode();
		try {
			Configurations.Freecam.save();
		} catch (IOException e) {
			Maucros.getLogger().warning("Failed to save Freecam settings!");
			Maucros.getLogger().catching(e);
		}
	}
	
	@Override
	public void onReval() {
		title.setPosition(width / 2, 15);
		back.setPosition(width / 2 - back.getWidth() / 2, 200);
		int xpl = width / 2 - 115;
		int xpb = xpl + 65;
		
		int y = 35;
		moveTitle.setPosition(width / 2, y);
		y += 25;
		moveDefault.setPosition(xpb, y);
		for (int i = 0; i < move.length; i++) {
			y += 25;
			move[i].setPosition(xpb, y);
			moveLabel[i].setPosition(xpl, y + 2);
		}
		y += 35;
		viewTitle.setPosition(width / 2, y);
		y += 25;
		viewMouse.setPosition(xpb, y);
		for (int i = 0; i < view.length; i++) {
			y += 25;
			view[i].setPosition(xpb, y);
			viewLabel[i].setPosition(xpl, y + 2);
		}
		
		if (Settings.Freecam.View.useMouse) setEnabled(false, view);
		else setEnabled(true, view);
		
		sb.setPosition(width - 100, 30);
		c.revalidate(0, 0, width, height);
		c2.revalidate(90, 30, width - 180, back.getY() - 35);
	}
	
	@Override
	protected void mouseClicked(int mx, int my, int code) {
		if (code == 0) {
			c.mouseClicked(mx, my);
			c2.mouseClicked(mx, my);
		}
	}
	
	@Override
	public void onButtonClicked(Button b) {
		if (b.equals(back)) close();
		else if (b.equals(viewMouse)) {
			Settings.Freecam.View.useMouse = viewMouse.getState() == 1;
			if (Settings.Freecam.View.useMouse) setEnabled(false, view);
			else setEnabled(true, view);
		}else if (b.equals(moveDefault)) {
			Settings.Freecam.Move.useDefault = moveDefault.getState() == 1;
			if (Settings.Freecam.Move.useDefault) setEnabled(false, view);
			else setEnabled(true, view);
		}
	}
	
	@Override
	public void onKeyTyped(char c, int kc) {
		if (!this.c2.keyTyped(c, kc)) this.unhandledKeyTyped(c, kc);
		checkFocus();
	}
	
	public void checkFocus() {
		if (c2.hasFocusedWidget()) try {
			for (FocusableButton b : move)
				if (!b.isFocused() && c2.getFocusedWidget().equals(b)) c2.setFocused(null);
			for (FocusableButton b : view)
				if (!b.isFocused() && c2.getFocusedWidget().equals(b)) c2.setFocused(null);
		} catch (Throwable t) {}
	}
}
