package net.maunium.Maucros.Gui.KeyMaucros;

import java.io.IOException;

import org.lwjgl.input.Keyboard;

import com.mcf.davidee.guilib.basic.FocusedContainer;
import com.mcf.davidee.guilib.basic.Label;
import com.mcf.davidee.guilib.core.Button;
import com.mcf.davidee.guilib.core.Container;
import com.mcf.davidee.guilib.vanilla.ButtonVanilla;
import com.mcf.davidee.guilib.vanilla.ScrollbarVanilla;

import net.maunium.Maucros.Maucros;
import net.maunium.Maucros.Config.Configurations;
import net.maunium.Maucros.Config.Settings;
import net.maunium.Maucros.Gui.MauScreen;
import net.maunium.Maucros.Gui.Config.GuiConfig;
import net.maunium.Maucros.Gui.Widgets.FixedFocusableLabel;
import net.maunium.Maucros.Gui.Widgets.ShiftableButton;
import net.maunium.Maucros.KeyMaucros.KeyMaucro;
import net.maunium.Maucros.Misc.I18n;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.Gui;

public class GuiKeyMaucros extends MauScreen {
	// kmlc means KeyMaucroListContainer
	private Container c, kmlc;
	private ScrollbarVanilla sb;
	private Label title;
	private ButtonVanilla newCC, newLua, back;
	private ShiftableButton[] buttons;
	private FixedFocusableLabel[] labels;
	
	public GuiKeyMaucros(GuiConfig parent) {
		super(parent);
	}
	
	@Override
	public void onReval() {
		Maucros.getLogger().trace("Setting positions...", this);
		title.setPosition(width / 2, 15);
		back.setPosition(width / 2 - back.getWidth() / 2 - newCC.getWidth() - 10, height - 40);
		newCC.setPosition(back.getX() + back.getWidth() + 5, height - 40);
		newLua.setPosition(newCC.getX() + newCC.getWidth() + 5, height - 40);
		if(!Settings.Enabled.lua) newLua.setEnabled(false);
		
		kmlc.getWidgets().clear();
		buttons = new ShiftableButton[Settings.KeyMaucros.getKeyMaucros().size()];
		labels = new FixedFocusableLabel[Settings.KeyMaucros.getKeyMaucros().size()];
		
		Maucros.getLogger().trace("Adding Key Maucros to KMLContainer", this);
		for (int i = 0; i < Settings.KeyMaucros.getKeyMaucros().size(); i++) {
			KeyMaucro km = Settings.KeyMaucros.getKeyMaucros().get(i);
			
			labels[i] = new FixedFocusableLabel(km.getName(), false);
			if (km.getShiftKeys().length <= 0) buttons[i] = new ShiftableButton(Keyboard.getKeyName(km.getKeyCode()), this);
			else buttons[i] = new ShiftableButton(km.getShiftKeysAsString() + " + " + Keyboard.getKeyName(km.getKeyCode()), this);
			buttons[i].setPosition(width / 2 - buttons[i].getWidth() / 2 + 22, 35 + 2 + i * 22);
			if(km.getType().equals(KeyMaucro.Type.LUA) && !Settings.Enabled.lua) buttons[i].setEnabled(false);
			labels[i].setPosition(width / 2 - buttons[i].getWidth() / 2 + 16 - labels[i].getWidth() + 2, 35 + 6 + i * 22);
		}
		Maucros.getLogger().trace("Adding key maucro widgets and setting scrollbar position", this);
		kmlc.addWidgets(buttons);
		kmlc.addWidgets(labels);
		
		sb.setPosition(width - 80, 35);
		Maucros.getLogger().trace("Revalidating containers", this);
		c.revalidate(0, 0, width, height);
		kmlc.revalidate(80, 35, width - 160, height - 83);
	}
	
	@Override
	public void onInit() {
		c = new Container();
		Maucros.getLogger().trace("Creating title label", this);
		title = new Label(I18n.format("conf.km.title"));
		Maucros.getLogger().trace("Creating buttons...", this);
		newCC = new ButtonVanilla(100, 20, I18n.format("conf.km.editor.cc.enter"), this);
		newLua = new ButtonVanilla(100, 20, I18n.format("conf.km.editor.lua.enter"), this);
		back = new ButtonVanilla(50, 20, I18n.format("conf.back"), this);
		
		Maucros.getLogger().trace("Creating KML container", this);
		sb = new ScrollbarVanilla(10);
		kmlc = new FocusedContainer(sb, 14, 4);
		Maucros.getLogger().trace("Adding widgets to main container", this);
		c.addWidgets(title, newCC, newLua, back);
		
		Maucros.getLogger().trace("Adding containers to list", this);
		containers.add(kmlc);
		containers.add(c);
		selectedContainer = c;
	}
	
	@Override
	public void drawBackground() {
		super.drawBackground();
		Gui.drawRect(kmlc.left(), kmlc.top(), kmlc.right() - 10, kmlc.bottom(), 0x44444444);
	}
	
	@Override
	public void save() {
		Maucros.getLogger().trace("Saving Key Maucros to file", this);
		Settings.KeyMaucros.sort();
		try {
			Configurations.KeyMaucros.save();
		} catch (IOException e) {
			Maucros.getLogger().warning("Failed to save Key Maucros!");
			Maucros.getLogger().catching(e);
		}
	}
	
	@Override
	public void onKeyTyped(char c, int kc) {
		super.onKeyTyped(c, kc);
		if (kc == Keyboard.KEY_DELETE) for (int i = 0; i < labels.length; i++)
			if (labels[i].isFocused()) {
				Settings.KeyMaucros.removeKeyMaucro(i);
				revalidateGui();
			}
	}
	
	@Override
	public void onButtonClicked(Button button) {
		if (button.equals(newCC)) Minecraft.getMinecraft().displayGuiScreen(new GuiAddKeyMaucro(this, false));
		else if (button.equals(newLua)) Minecraft.getMinecraft().displayGuiScreen(new GuiAddKeyMaucro(this, true));
		else if (button.equals(back)) close();
		else for (int i = 0; i < Settings.KeyMaucros.getKeyMaucros().size(); i++)
			if (Settings.KeyMaucros.getKeyMaucros().get(i).getName().equals(labels[i].getText()) && buttons[i].equals(button)) {
				KeyMaucro km = Settings.KeyMaucros.getKeyMaucros().get(i);
				GuiAddKeyMaucro gakm = new GuiAddKeyMaucro(this, km.getType().equals(KeyMaucro.Type.LUA));
				Minecraft.getMinecraft().displayGuiScreen(gakm);
				gakm.addValues(km.getName(), km.getData(), km.getKeyCode(), km.getExecutionPhase(), i, km.getShiftKeys());
				break;
			}
	}
	
	// public void addKeyMaucro(String name, String chat, int keycode, int... shiftKeys) {
	// Settings.KeyMaucros.addKeyMaucro(new KeyMaucro(name.trim(), chat, keycode, shiftKeys));
	// }
	//
	// public void setKeyMaucro(String name, String chat, int keycode, int index, int... shiftKeys) {
	// Settings.KeyMaucros.modifyKeyMaucro(index, new KeyMaucro(name.trim(), chat, keycode, shiftKeys));
	// }
}
