package net.maunium.Maucros.Gui;

import org.lwjgl.input.Keyboard;

import com.mcf.davidee.guilib.basic.BasicScreen;
import com.mcf.davidee.guilib.core.Button;
import com.mcf.davidee.guilib.core.Button.ButtonHandler;

import net.maunium.Maucros.Maucros;
import net.maunium.Maucros.Config.Settings;
import net.maunium.Maucros.Gui.Widgets.FocusableButton;
import net.maunium.Maucros.Misc.MaucrosLicensor;

import net.minecraft.client.gui.GuiScreen;

/**
 * An extended BasicScreen.
 * 
 * @author Tulir293
 * @since 1.4 Alpha 3
 */
public abstract class MauScreen extends BasicScreen implements ButtonHandler {
	/**
	 * Constructs a new MauScreen and sets the given GuiScreen as the parent.
	 */
	public MauScreen(GuiScreen parent) {
		super(parent);
	}
	
	/**
	 * Called from createGui after other necessary tasks have been completed.
	 */
	public abstract void onInit();
	
	/**
	 * Called from revalidateGui after other necessary tasks have been completed.
	 */
	public abstract void onReval();
	
	/**
	 * Called from reopenedGui after other necessary tasks have been completed.<br>
	 * It is not necessary to override this, as I've found it to be quite useless.
	 */
	public void onReopen() {};
	
	/**
	 * Called from buttonClicked after other necessary tasks have been completed<br>
	 * It is not necessary to override this, as sometimes a GUI may not have buttons.
	 */
	public void onButtonClicked(Button b) {}
	
	/**
	 * Called from keyTyped after other necessary tasks have been completed<br>
	 * It is not necessary to override this, as sometimes a GUI may not have key actions.
	 */
	public void onKeyTyped(char c, int kc) {
		try {
			super.keyTyped(c, kc);
		} catch (Throwable t) {}
	}
	
	/**
	 * Called from close after other necessary tasks have been completed<br>
	 * It is not necessary to override this, as sometimes a GUI may not have anything special to do on close
	 * 
	 * @return True if save() should be called too.
	 */
	public boolean onClose() {
		return true;
	}
	
	/**
	 * Called from close() if config smart save is enabled and onClose returns true.
	 */
	public void save() {}
	
	public void setEnabled(boolean enabled, Button... ws) {
		for (Button w : ws) {
			w.setEnabled(enabled);
		}
	}
	
	public void setEnabled(boolean enabled, FocusableButton... ws) {
		for (FocusableButton w : ws) {
			w.setEnabled(enabled);
		}
	}
	
	@Override
	public final void createGui() {
		if (!MaucrosLicensor.check()) return;
		Maucros.getLogger().debug("Creating GUI", this);
		onInit();
	}
	
	@Override
	public final void revalidateGui() {
		if (!MaucrosLicensor.check()) return;
		Maucros.getLogger().debug("Revalidating GUI", this);
		onReval();
	}
	
	@Override
	public final void reopenedGui() {
		if (!MaucrosLicensor.check()) return;
		Maucros.getLogger().debug("Reopened GUI", this);
		onReopen();
	}
	
	@Override
	public final void close() {
		Maucros.getLogger().debug("Closing GUI", this);
		if (onClose() && Settings.Enabled.autosave) save();
		if (MaucrosLicensor.check()) super.close();
	}
	
	@Override
	public final void keyTyped(char c, int kc) {
		if (!MaucrosLicensor.check()) return;
		Maucros.getLogger().trace("Key typed: " + c + ", " + kc + ", " + Keyboard.getKeyName(kc), this);
		if (kc == Keyboard.KEY_ESCAPE) close();
		onKeyTyped(c, kc);
	}
	
	@Override
	public final void buttonClicked(Button b) {
		Maucros.getLogger().trace("Button clicked: " + b.getText(), this);
		onButtonClicked(b);
	}
	
	@Override
	public boolean doesGuiPauseGame() {
		return true;
	}
}
