package net.maunium.Maucros.Listeners;

import java.nio.charset.StandardCharsets;
import java.util.regex.Pattern;

import org.apache.commons.codec.binary.Base64;
import org.jasypt.encryption.StandardPBEStringEncryptor;
import org.jasypt.salt.StringFixedSaltGenerator;

import net.maunium.Maucros.Maucros;
import net.maunium.Maucros.Config.Settings;
import net.maunium.Maucros.Misc.Calculator;
import net.maunium.Maucros.Misc.I18n;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiNewChat;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.ChatStyle;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.IChatComponent;

import net.minecraftforge.client.event.ClientChatReceivedEvent;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

public class InChatListener {
	// The previous message.
	private String lastMessage = "";
	// The count of similiar messages in a row.
	private int countOfSpam = 1;
	// The message id (or whatever the Chat GUI calls it) of the previous message.
	private int prevId = 0;
	
	@SubscribeEvent(priority = EventPriority.HIGHEST)
	public void onChat(ClientChatReceivedEvent evt) {
		if (Settings.disabled) return;
		// Cancels the event.
		evt.setCanceled(true);
//		if (Settings.MauChat.onlyShowMauchat) return;
		
		// Gets the chat instance.
		GuiNewChat gnc = Minecraft.getMinecraft().ingameGUI.getChatGUI();
		// Gets the message sent.
		String message = evt.message.getUnformattedText();
		
		try {
			if (message.contains("TUFVQjY0")) {
				String[] tmp = evt.message.getFormattedText().split(Pattern.quote("TUFVQjY0"), 2);
				tmp[1] = tmp[1].substring(0, tmp[1].length() - 2);
				String decoded = new String(Base64.decodeBase64(tmp[1].getBytes(StandardCharsets.UTF_8)), StandardCharsets.UTF_8);
				
				evt.message = new ChatComponentText(tmp[0]);
				evt.message.appendText(decoded.replace("&", "§"));
				evt.message.appendSibling(new ChatComponentText(" [B64]").setChatStyle(new ChatStyle().setColor(EnumChatFormatting.GRAY).setItalic(true)));
				
				message = EnumChatFormatting.getTextWithoutFormattingCodes(evt.message.getUnformattedText());
				Maucros.getChatLogger().custom(tmp[1] + " -> " + decoded, "IN/BASE64", true);
			} else if (message.contains("B5D3S")) {
				String[] tmp = evt.message.getFormattedText().split(Pattern.quote("B5D3S"), 2);
				tmp[1] = tmp[1].substring(0, tmp[1].length() - 2);
				StandardPBEStringEncryptor enc = new StandardPBEStringEncryptor();
				enc.setStringOutputType(Settings.Encryption.base64 ? "base64" : "hexadecimal");
				enc.setPassword(Settings.Encryption.password);
				enc.setSaltGenerator(new StringFixedSaltGenerator(Settings.Encryption.salt));
				enc.setAlgorithm("PBEWithMD5AndTripleDES");
				
				String decrypted;
				try {
					decrypted = enc.decrypt(tmp[1]);
				} catch (Exception e) {
					Maucros.getLogger().error("Failed to decrypt message: " + tmp[1] + ".");
					Maucros.getLogger().catching(e);
					return;
				}
				evt.message = new ChatComponentText(tmp[0]);
				evt.message.appendText(decrypted.replace("&", "§"));
				evt.message.appendSibling(new ChatComponentText(" [ENC]").setChatStyle(new ChatStyle().setColor(EnumChatFormatting.GRAY).setItalic(true)));
				
				message = EnumChatFormatting.getTextWithoutFormattingCodes(evt.message.getUnformattedText());
				Maucros.getChatLogger().custom(tmp[1] + " -> " + decrypted, "IN/CRYPT", true);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (Settings.Enabled.calculatorHost) {
			try {
				if (message.toLowerCase().contains("#calc")) {
					String[] tmp = message.split("(?i)" + Pattern.quote("#calc"));
					String s = Calculator.processCalculation(tmp[1]);
					if (s.equals("varname")) Maucros.sendChat(I18n.format("message.calculator.variable.spacename"));
					else if (s.equals("syntax")) Maucros.sendChat("A wild Syntax Exception appears!");
					else Maucros.sendChat(s);
				}
			} catch (Throwable e) {}
		}
		
		// Is this message same as the previous one?
		if (Settings.Enabled.spamfilter && message.equalsIgnoreCase(lastMessage)) {
			// Increase count of spam variable.
			countOfSpam++;
			// This is the complicated part...
			// Prints a chat message and deletes the previous one.
			IChatComponent spamCount = new ChatComponentText(" [" + countOfSpam + "]").setChatStyle(new ChatStyle().setColor(EnumChatFormatting.GRAY)
					.setItalic(true));
			evt.message = evt.message.appendSibling(spamCount);
			gnc.setChatLine(evt.message, prevId, Minecraft.getMinecraft().ingameGUI.getUpdateCounter(), false);
		} else {
			// Increase the id by one
			prevId++;
			// Set spam count to 1 (AKA. No spam)
			countOfSpam = 1;
			// Print chat message and set it's id to prevId.
			gnc.setChatLine(evt.message, prevId, Minecraft.getMinecraft().ingameGUI.getUpdateCounter(), false);
			Maucros.getChatLogger().in(message);
		}
		// Set the last message to this message.
		lastMessage = message;
	}
}
