package net.maunium.Maucros.MauChat;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;

import net.maunium.MauChat.Packet;
import net.maunium.Maucros.Maucros;
import net.maunium.Maucros.Logging.ChatLogger;

import net.minecraft.util.ChatComponentText;
import net.minecraft.util.ChatStyle;
import net.minecraft.util.EnumChatFormatting;

/**
 * An adapted version of the MauChat ChatClient class
 * @author Tulir293
 * @since Unknown 1.2 developement version
 */
public class AdaptedChatClient {
	public final String server;
	public final int port;
	private OutputStream out;
	private InputStream in;
	private Socket connection;
	private boolean authenticated;
	private AdaptedSocketReader sr;
	public static final String version = "1.0";
	
	public AdaptedChatClient(String server, int port) {
		this.server = server;
		this.port = port;
		authenticated = false;
	}
	
	public void start() throws UnknownHostException, IOException {
		AdaptedInputReader.create();
		connect();
	}
	
	public void connect() throws UnknownHostException, IOException {
		connection = new Socket(server, port);
		out = connection.getOutputStream();
		in = connection.getInputStream();
		out.write(Packet.JOIN_STANDARD);
	}
	
	public void close() throws IOException {
		out.close();
		in.close();
		connection.close();
		sr.stopThread();
	}
	
	protected void setAuthenticated() {
		authenticated = true;
		sr = new AdaptedSocketReader();
		sr.start();
	}
	
	public OutputStream getOut() {
		return out;
	}
	
	public InputStream getIn() {
		return in;
	}
	
	public final boolean isAuthenticated() {
		return authenticated;
	}
	
	public void print(Object msg) {
		ChatLogger.printChatNoLog(new ChatComponentText(msg.toString()).setChatStyle(new ChatStyle().setColor(EnumChatFormatting.WHITE)));
		Maucros.getChatLogger().custom(msg, "Mau/OUT", true);
	}
	
	public void println(Object msg) {
		print(msg);
	}
	
	public void error(Object msg) {
		ChatLogger.printChatNoLog(new ChatComponentText(msg.toString()).setChatStyle(new ChatStyle().setColor(EnumChatFormatting.RED)));
		Maucros.getChatLogger().custom(msg, "Mau/ERR", true);
	}
	
	public void errorln(Object msg) {
		error(msg);
	}
	
	public void debugln(Object msg) {
		Maucros.getLogger().debug(msg);
	}
}
