package net.maunium.Maucros.MauChat;

import net.maunium.MauChat.Packet;
import net.maunium.Maucros.Config.Settings;

/**
 * An adapted version of the MauChat InputReader class
 * @author Tulir293
 * @since Unknown 1.2 developement version
 */
public class AdaptedInputReader {
	private AdaptedChatClient host;
	private static AdaptedInputReader instance;
	
	private AdaptedInputReader() {
		host = Settings.MauChat.getInstance();
	}
	
	public static void create() {
		AdaptedInputReader.instance = new AdaptedInputReader();
	}
	
	public static AdaptedInputReader getInstance() {
		return AdaptedInputReader.instance;
	}
	
	public void handleChat(String msg) {
		if (!host.isAuthenticated()) host.println("Log in first with /login!");
		else if (!msg.trim().equals("")) new Packet(Packet.CHAT, msg).tryWrite(host.getOut());
	}
	
	public void handleCommand(String msg) {
		if (!host.isAuthenticated()) {
			if (msg.toLowerCase().startsWith("login")) {
				if (msg.length() > 6 && msg.contains(" ")) {
					host.debugln("Begin login sequence...");
					String tmp1 = msg.substring(msg.indexOf(" ") + 1, msg.lastIndexOf(" "));
					String tmp2 = msg.substring(msg.lastIndexOf(" ") + 1);
					byte[] name, password;
					host.debugln("Serializing name and password...");
					name = Packet.serialize(tmp1);
					password = Packet.serialize(tmp2);
					host.debugln("Attempting to write name request.");
					Packet nameRequest = new Packet(Packet.LOGIN_USERNAME, name);
					host.debugln("Succeeded: " + nameRequest.tryWrite(host.getOut()));
					
					host.debugln("Attempting to read name response.");
					Packet nameResponse = Packet.tryReadPacket(host.getIn());
					host.debugln("Done, response:");
					if (nameResponse.id == Packet.USERNAME_DENIED) {
						host.errorln(nameResponse.getData());
						return;
					} else if (nameResponse.id == Packet.USERNAME_ACCEPTED) host.println(nameResponse.getData());
					
					host.debugln("Attempting to write password request.");
					Packet passwdRequest = new Packet(Packet.LOGIN_PASSWORD, password);
					passwdRequest.tryWrite(host.getOut());
					
					host.debugln("Attempting to read password response.");
					Packet passwdResponse = Packet.tryReadPacket(host.getIn());
					if (passwdResponse.id == Packet.PASSWORD_ACC_LOGIN) {
						host.println("Password accepted. Successfully logged in as " + passwdResponse.getData() + ".");
						host.setAuthenticated();
					} else if (passwdResponse.id == Packet.PASSWORD_ACC_REGISTER) {
						host.println("Password accepted. Successfully registered as " + passwdResponse.getData() + ".");
						host.setAuthenticated();
					} else if (passwdResponse.id == Packet.PASSWORD_DENIED) {
						host.errorln(passwdResponse.getData());
						return;
					}
					host.debugln("End login sequence.");
				} else host.println("Usage: /login <username> <password>");
			} else host.println("Log in first with /login!");
		} else if (msg.toLowerCase().startsWith("exit") || msg.toLowerCase().startsWith("quit") || msg.toLowerCase().startsWith("logout")) Settings.MauChat
				.close();
		else new Packet(Packet.COMMAND, msg).tryWrite(host.getOut());
	}
}
