package net.maunium.Maucros;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Arrays;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.google.common.io.ByteStreams;

import net.maunium.Maucros.Actions.Actions;
import net.maunium.Maucros.Config.Configurations;
import net.maunium.Maucros.Config.Settings;
import net.maunium.Maucros.Gui.Widgets.StateButton;
import net.maunium.Maucros.KeyMaucros.CCKeyMaucro;
import net.maunium.Maucros.KeyMaucros.Commands.CommandDelay;
import net.maunium.Maucros.KeyMaucros.Commands.CommandForward;
import net.maunium.Maucros.KeyMaucros.Commands.CommandLocal;
import net.maunium.Maucros.KeyMaucros.Commands.CommandLook;
import net.maunium.Maucros.Listeners.ConnectListener;
import net.maunium.Maucros.Listeners.GuiOpenListener;
import net.maunium.Maucros.Listeners.InChatListener;
import net.maunium.Maucros.Listeners.KeyInputListener;
import net.maunium.Maucros.Listeners.OutChatListener;
import net.maunium.Maucros.Listeners.OverlayRenderListener;
import net.maunium.Maucros.Listeners.PacketListener;
import net.maunium.Maucros.Listeners.TickListener;
import net.maunium.Maucros.Listeners.WorldRenderListener;
import net.maunium.Maucros.Logging.ChatLogger;
import net.maunium.Maucros.Logging.MaucrosLogger;
import net.maunium.Maucros.Misc.I18n;
import net.maunium.Maucros.Misc.MauKeyBinds;
import net.maunium.Maucros.Misc.MaucrosLicensor;

import net.minecraft.client.Minecraft;
import net.minecraft.crash.CrashReport;
import net.minecraft.crash.CrashReportCategory;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.ChatStyle;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.IChatComponent;
import net.minecraft.util.ReportedException;

import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fml.common.FMLCommonHandler;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.Mod.EventHandler;
import net.minecraftforge.fml.common.Mod.Instance;
import net.minecraftforge.fml.common.Mod.InstanceFactory;
import net.minecraftforge.fml.common.ModMetadata;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLModDisabledEvent;
import net.minecraftforge.fml.common.event.FMLPostInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.network.NetworkRegistry;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

/**
 * The main class for Maucros. This is the only fully documented class in Maucros...<br>
 * I guess that's not actually such a good thing.
 * 
 * @author Tulir293
 * @since 0.1
 * @version 1.5 Alpha 7. Same as {@link #version}
 */
@SideOnly(Side.CLIENT)
@Mod(modid = Maucros.name, version = Maucros.version, name = Maucros.name, acceptedMinecraftVersions = "[1.8,1.9)", canBeDeactivated = true)
public class Maucros {
	/*
	 * Start of class. Start of field declarations.
	 */
	/** A singleton instance of Maucros */
	@Instance(Maucros.name)
	private static Maucros instance;
	/** Manually updated to match the version in development. */
	public static final String version = "1.5 Alpha 7";
	public static final String ksv = "1.5";
	/** Most probably will never change */
	public static final String name = "Maucros";
	/** The color of the standard output messages */
	public static final ChatStyle stdStyle = new ChatStyle().setColor(EnumChatFormatting.GRAY);
	/** A colorful IChatComponent that should be used as the start of each Maucros standard output message. */
	public static final IChatComponent stdTag = new ChatComponentText("")
			.appendSibling(new ChatComponentText("[").setChatStyle(new ChatStyle().setColor(EnumChatFormatting.AQUA)))
			.appendSibling(new ChatComponentText(Maucros.name).setChatStyle(new ChatStyle().setColor(EnumChatFormatting.GREEN).setBold(true)))
			.appendSibling(new ChatComponentText("]").setChatStyle(new ChatStyle().setColor(EnumChatFormatting.AQUA)))
			.appendSibling(new ChatComponentText(" ").setChatStyle(stdStyle));
	/** The color of the error output messages */
	public static final ChatStyle errStyle = new ChatStyle().setColor(EnumChatFormatting.RED);
	/** A colorful IChatComponent that should be used as the start of each Maucros error output message. */
	public static final IChatComponent errTag = new ChatComponentText("")
			.appendSibling(new ChatComponentText("[").setChatStyle(new ChatStyle().setColor(EnumChatFormatting.DARK_PURPLE)))
			.appendSibling(new ChatComponentText(Maucros.name).setChatStyle(new ChatStyle().setColor(EnumChatFormatting.DARK_RED).setBold(true)))
			.appendSibling(new ChatComponentText("]").setChatStyle(new ChatStyle().setColor(EnumChatFormatting.DARK_PURPLE)))
			.appendSibling(new ChatComponentText(" ").setChatStyle(errStyle));
	/** A standard Minecraft logger with the name "Maucros" ({@link #name}) */
	private Logger log;
	/** Singleton Instance of {@link Actions} */
	private Actions actions;
	/** Singleton Instance of {@link MauKeyBinds} */
	private MauKeyBinds mauKeyBinds;
	/** Singleton Instance of {@link KeyInputListener} */
	private KeyInputListener keyInputListener;
	/** Singleton Instance of {@link TickListener} */
	private TickListener tickListener;
	/** Singleton Instance of {@link OverlayRenderListener} */
	private OverlayRenderListener overlayRenderListener;
	/** Singleton Instance of {@link WorldRenderListener} */
	private WorldRenderListener worldRenderListener;
	/** Singleton Instance of {@link InChatListener} */
	private InChatListener inChatListener;
	/** Singleton Instance of {@link OutChatListener} */
	private OutChatListener outChatListener;
	/** Singleton Instance of {@link GuiOpenListener} */
	private GuiOpenListener guiOpenListener;
	/** Singleton Instance of {@link ConnectListener} */
	private ConnectListener connectListener;
	/** Singleton Instance of {@link PacketListener} */
	private PacketListener packetListener;
	
	/** The real out channel since Forge overrides System.out */
	public static PrintStream sout = new PrintStream(new FileOutputStream(FileDescriptor.out));
	/** The real err channel since Forge overrides System.err */
	public static PrintStream serr = new PrintStream(new FileOutputStream(FileDescriptor.err));
	// Init times in milliseconds.
	private int construct = -1, preInit = -1, init = -1, postInit = -1;
	
	/*
	 * End of field declarations. Init methods go in order Construct -> preInit -> init -> postInit -> ... -> deactivate. Start of init methods.
	 */
	
	private Maucros() {
		// If Maucros is not licenced on this device or it has been killswitched, return
		if (!MaucrosLicensor.check()) return;
		instance = this;
		long st = Minecraft.getSystemTime();
		log = LogManager.getLogger("Maucros");
		log.info("Loading Maucros Logger (see logs/Maucros/ for log files)");
		// Create the Maucros Logger.
		MaucrosLogger.create();
		// From here on, the debug and trace logger calls describe everything.
		getLogger().debug("Maucros Logger opened from constructor in " + (construct = (int) (Minecraft.getSystemTime() - st)) + "ms.");
		getLogger().info("Beginning initialization of Maucros v" + version + ".");
	}
	
	@EventHandler
	public void preInit(FMLPreInitializationEvent evt) {
		if (!MaucrosLicensor.check()) return;
		long st = Minecraft.getSystemTime();
		
		Maucros.getLogger().debug("PreInit begin.");
		Maucros.getLogger().trace("Loading chat logger (see /logs/Chat/ for files)");
		ChatLogger.create();
		loadConfig();
		checkSoftDeps();
		setModMetadata(evt.getModMetadata());
		
		long et = Minecraft.getSystemTime();
		Maucros.getLogger().info("PreInit complete in " + (preInit = (int) (et - st)) + "ms.");
	}
	
	@EventHandler
	public void init(FMLInitializationEvent evt) {
		if (!MaucrosLicensor.check()) return;
		Maucros.getLogger().debug("Init begin.");
		long st = Minecraft.getSystemTime();
		
		reloadLanguage();
		
		Maucros.getLogger().debug("Creating instances...");
		Maucros.getLogger().trace("Creating instance of Actions...");
		actions = Actions.create();
		Maucros.getLogger().trace("Creating instance of MauKeyBinds");
		mauKeyBinds = new MauKeyBinds();
		
		Maucros.getLogger().debug("Creating and registering MaucroCommands");
		Maucros.getLogger().trace("Creating and Registering command Local");
		CCKeyMaucro.registerMaucroCommand(new CommandLocal());
		Maucros.getLogger().trace("Creating and Registering command Look");
		CCKeyMaucro.registerMaucroCommand(new CommandLook());
		Maucros.getLogger().trace("Creating and Registering command Forward");
		CCKeyMaucro.registerMaucroCommand(new CommandForward());
		Maucros.getLogger().trace("Creating and Registering command Delay");
		CCKeyMaucro.registerMaucroCommand(new CommandDelay());
		
		// FML events
		Maucros.getLogger().debug("Creating and Registering Listeners");
		Maucros.getLogger().trace("Creating and Registering FML Event Listeners");
		Maucros.getLogger().trace("Creating and Registering TickListener");
		FMLCommonHandler.instance().bus().register(tickListener = new TickListener(this));
		Maucros.getLogger().trace("Creating and Registering Key Binding listener");
		FMLCommonHandler.instance().bus().register(keyInputListener = new KeyInputListener(this));
		Maucros.getLogger().trace("Creating and Registering ConnectListener");
		FMLCommonHandler.instance().bus().register(connectListener = new ConnectListener());
		Maucros.getLogger().trace("Creating and Registering PacketListener");
		NetworkRegistry.INSTANCE.newEventDrivenChannel("Maucros").register(packetListener = new PacketListener());
		
		// Forge events
		Maucros.getLogger().trace("Creating and Registering Forge Event Listeners");
		Maucros.getLogger().trace("Creating and Registering OverlayRenderListener");
		MinecraftForge.EVENT_BUS.register(overlayRenderListener = new OverlayRenderListener(this));
		Maucros.getLogger().trace("Creating and Registering WorldRenderListener");
		MinecraftForge.EVENT_BUS.register(worldRenderListener = new WorldRenderListener());
		Maucros.getLogger().trace("Creating and Registering InChatListener");
		MinecraftForge.EVENT_BUS.register(inChatListener = new InChatListener());
		Maucros.getLogger().trace("Creating and Registering GuiOpenListener");
		MinecraftForge.EVENT_BUS.register(guiOpenListener = new GuiOpenListener());
		
		if (Settings.Enabled.mauevtlib) {
//			Maucros.getLogger().debug("MauChat is enabled. Loading config and registering outgoing chat listener.");
//			try {
//				Configurations.MauChat.load();
//			} catch (IOException e) {
//				Maucros.getLogger().warning("Failed to load MauChat configuration!");
//				Maucros.getLogger().catching(e);
//			}
			MinecraftForge.EVENT_BUS.register(outChatListener = new OutChatListener());
		}
		
		long et = Minecraft.getSystemTime();
		Maucros.getLogger().info("Init complete in " + (init = (int) (et - st)) + "ms.");
	}
	
	@EventHandler
	public void postInit(FMLPostInitializationEvent evt) {
		if (!MaucrosLicensor.check()) return;
		
		Maucros.getLogger().debug("PostInit begin.");
		long st = Minecraft.getSystemTime();
		
		if (!Loader.isModLoaded("maueventlib") && Settings.Enabled.mauevtlib) {
			Settings.Enabled.mauevtlib = false;
			Maucros.getLogger().warning("MauEventLib seems to be installed, but doesn't seem to be loaded.");
		}
		if (!Loader.isModLoaded("guilib") && Settings.Enabled.gui) {
			Settings.Enabled.gui = false;
			Maucros.getLogger().warning("GuiLib seems to be installed, but doesn't seem to be loaded. Disabling Config GUI.");
		}
		
		long et = Minecraft.getSystemTime();
		Maucros.getLogger().info("PostInit complete in " + (postInit = (int) (et - st)) + "ms.");
		Maucros.getLogger().info("All done! Maucros v" + version + " enabled in " + (construct + preInit + init + postInit) + "ms.");
	}
	
	@EventHandler
	public void deactivate(FMLModDisabledEvent evt) {
		if (!MaucrosLicensor.check()) return;
		long st = Minecraft.getSystemTime();
		Maucros.getLogger().info("Disabling Maucros...");
		if (Settings.Enabled.lua) {
			try {
				Maucros.getLogger().debug("Saving Lua Global Data...");
				mauluam.MauGDataLib.saveGlobalData();
			} catch (IOException e) {
				Maucros.getLogger().warning("Failed to save Lua Global Data!");
				Maucros.getLogger().catching(e);
			}
		}
		
		if (Settings.Enabled.lua) mauluam.MauThreadLib.stopThreads();
		
		Maucros.getLogger().debug("Unregistering listeners...");
		FMLCommonHandler.instance().bus().unregister(tickListener);
		FMLCommonHandler.instance().bus().unregister(mauKeyBinds);
		FMLCommonHandler.instance().bus().unregister(connectListener);
		MinecraftForge.EVENT_BUS.unregister(overlayRenderListener);
		MinecraftForge.EVENT_BUS.unregister(inChatListener);
		MinecraftForge.EVENT_BUS.unregister(guiOpenListener);
		MinecraftForge.EVENT_BUS.unregister(worldRenderListener);
		if (Settings.Enabled.mauevtlib) MinecraftForge.EVENT_BUS.unregister(outChatListener);
		
		Maucros.getLogger().info("Closing Chat Logger...");
		try {
			Maucros.getChatLogger().close();
		} catch (IOException e1) {
			Maucros.getLogger().catching(e1);
		}
		
		Maucros.getLogger().info("Closing Maucros Logger... Wait, that's me?! Oh shit...");
		try {
			Maucros.getLogger().close();
		} catch (IOException e) {
			log.catching(e);
		}
		long et = Minecraft.getSystemTime();
		log.info("Maucros Disabled. Total time: " + (int) (et - st) + "ms.");
	}
	
	/*
	 * End of init methods. Start of chat methods.
	 */
	
	/**
	 * Localizes a message with I18n and prints it to chat.
	 * 
	 * @param key The I18n tag
	 * @param args The I18n args
	 */
	public static void printChatLocalized(String key, Object... args) {
		ChatLogger.printChat(stdTag.createCopy().appendText(I18n.format(key, args)).setChatStyle(stdStyle));
	}
	
	/**
	 * Localizes a message with I18n and prints it to chat as an error.
	 * 
	 * @param key The I18n tag
	 * @param args The I18n args
	 */
	public static void printChatErrorLocalized(String key, Object... args) {
		ChatLogger.printChat(errTag.createCopy().appendText(I18n.format(key, args)).setChatStyle(errStyle));
	}
	
	/**
	 * Localizes a message with I18n and prints it to chat using the given style.
	 * 
	 * @param style The Chat Style to use.
	 * @param key The I18n tag
	 * @param args The I18n args
	 */
	public static void printChatStyledLocalized(ChatStyle style, String message, Object... args) {
		ChatLogger.printChat(new ChatComponentText(I18n.format(message, args)).setChatStyle(style));
	}
	
	/**
	 * Prints a chat message (without sending to server)<br>
	 * Uses the public static final String stag as prefix and gray as message color
	 * 
	 * @param message The message to print.
	 * @deprecated Use {@link #printChatLocalized}
	 */
	@Deprecated
	public static void printChat(String message) {
		ChatLogger.printChat(stdTag.createCopy().appendText(message).setChatStyle(stdStyle));
	}
	
	/**
	 * Prints a chat error message (without sending to server)<br>
	 * Uses the public static final String errtag as prefix and red as message color
	 * 
	 * @param message The message to print.
	 * @deprecated Use {@link #printChatErrorLocalized}
	 */
	@Deprecated
	public static void printChatError(String message) {
		ChatLogger.printChat(errTag.createCopy().appendText(message).setChatStyle(errStyle));
	}
	
	/**
	 * Prints a styled chat message (without sending to server)<br>
	 * There is no prefix and all color is specified in the message and/or the ChatStyle object.
	 * 
	 * @param message The message to print.
	 * @param style The ChatStyle object to apply to the message.
	 * @deprecated Use {@link #printChatStyledLocalized}
	 */
	@Deprecated
	public static void printChatStyled(Object message, ChatStyle style) {
		ChatLogger.printChat(new ChatComponentText(message.toString()).setChatStyle(style));
	}
	
	/**
	 * Sends a chat message to the server. Color support is up to the server.<br>
	 * The server has the ability to modify this all it wants.
	 * 
	 * @param message The message to send
	 */
	public static void sendChat(String message) {
		if (Minecraft.getMinecraft() == null) return;
		if (Minecraft.getMinecraft().thePlayer == null) return;
		Minecraft.getMinecraft().thePlayer.sendChatMessage(message);
	}
	
	/*
	 * End of chat methods. Start of language methods.
	 */
	
	/**
	 * Load the language file set in Settings.language or gamesettings.language<br>
	 * If the given language file doesn't exist, defaults to en_US.lang<br>
	 * If en_US.lang doesn't exist and copying it from the jar to disk fails, Minecraft will crash.
	 */
	public static void reloadLanguage() {
		String language = null;
		try {
			if (Settings.language != null && !Settings.language.isEmpty()) {
				language = Settings.language;
				Maucros.getLogger().info("Loading Maucros-specific language: " + language);
			} else {
				language = Minecraft.getMinecraft().gameSettings.language;
				Maucros.getLogger().info("Loading Minecraft language: " + language);
			}
			
			I18n.init(Configurations.getDirectory("lang"), language);
			
			refreshStaticI18n();
		} catch (IOException e) {
			CrashReport crash = CrashReport.makeCrashReport(e, "Loading Maucros Language");
			CrashReportCategory sizes = crash.makeCategory("Loading Maucros Language");
			sizes.addCrashSection("Selected Language", language);
			sizes.addCrashSection("Language Directory", Configurations.getDirectory("lang"));
			throw new ReportedException(crash);
		}
	}
	
	/**
	 * Refreshes static Strings that by default only call I18n format once.
	 */
	public static void refreshStaticI18n() {
		StateButton.DefaultFormat.refreshStaticI18n();
	}
	
	/**
	 * Saves all premade language files.
	 */
	public static void saveLangFiles() throws IOException {
		saveLangFile("en_US");
		saveLangFile("fi_FI");
	}
	
	/**
	 * Copies a language file from the lang folder inside the jar to the folder in Configurations.getDirectory("lang").
	 * 
	 * @param lang The name of the language file without the ending.
	 * @throws IOException If reading (Resource as Stream) or writing (bytestreams.copy) fails
	 */
	private static void saveLangFile(String lang) throws IOException {
		File f = new File(Configurations.getDirectory("lang"), lang + ".lang");
		if (f.exists()) f.delete();
		InputStream input = Maucros.class.getResourceAsStream("/lang/" + lang + ".lang");
		OutputStream output = new FileOutputStream(f);
		ByteStreams.copy(input, output);
		output.flush();
		output.close();
		input.close();
	}
	
	/*
	 * End of language methods. Start of config/misc methods.
	 */
	
	/**
	 * Checks if soft dependencies are installed and changes Settings accordingly.
	 */
	public void checkSoftDeps() {
		Maucros.getLogger().debug("Checking soft dependencies...");
		
		Maucros.getLogger().trace("Checking if MauEventLib is installed...");
		try {
			Maucros.getLogger().trace("Class " + Class.forName("net.maunium.MauEventLib.ClientChatSendEvent").getSimpleName() + " was found.");
			Settings.Enabled.mauevtlib = true;
		} catch (Exception e) {
			Settings.Enabled.mauevtlib = false;
			Maucros.getLogger().warning("MauEventLib doesn't seem to be installed.");
		}
		Maucros.getLogger().trace("Checking if GuiLib is installed...");
		try {
			Maucros.getLogger().trace("Class " + Class.forName("com.mcf.davidee.guilib.GuiLibrary").getSimpleName() + " was found. Enabling Config GUI.");
			Settings.Enabled.gui = true;
		} catch (Exception e) {
			Settings.Enabled.gui = false;
			Maucros.getLogger().warning("GuiLib doesn't seem to be installed. Disabling Config GUI.");
		}
		Maucros.getLogger().trace("Checking if LuaJ API is available...");
		try {
			Maucros.getLogger().trace("Class " + Class.forName("org.luaj.vm2.Lua").getSimpleName() + " was found. Enabling Lua Key Maucro support.");
			Settings.Enabled.lua = true;
		} catch (Exception e) {
			Settings.Enabled.lua = false;
			Maucros.getLogger().warning("LuaJ API doesn't seem to be installed. Disabling Lua Key Maucro support.");
		}
	}
	
	/**
	 * Loads all configurations.
	 */
	public void loadConfig() {
		try {
			Maucros.getLogger().info("Loading configurations");
			Configurations.Generic.load();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to load configurations!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Loading Friend list");
			Configurations.Friends.load();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to load friend list!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Loading Target list");
			Configurations.Targets.load();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to load target list!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Loading Key Maucros");
			Configurations.KeyMaucros.load();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to load key maucros!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Loading Alts");
			Configurations.Alts.load();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to load alts!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Loading X-Ray block config");
			Configurations.XRay.load();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to load X-Ray block config!");
			Maucros.getLogger().catching(e);
		}
		if (Settings.Enabled.lua) {
			try {
				Maucros.getLogger().trace("Loading Lua Global Data");
				mauluam.MauGDataLib.loadGlobalData();
			} catch (Throwable e) {
				Maucros.getLogger().warning("Failed to load Lua Global Data!");
				Maucros.getLogger().catching(e);
			}
		}
		try {
			Maucros.getLogger().trace("Saving premade language files");
			saveLangFiles();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to save premade language files!");
			Maucros.getLogger().catching(e);
		}
	}
	
	/**
	 * Saves all configurations.
	 */
	public void saveConfig() {
		try {
			Maucros.getLogger().info("Saving configurations");
			Configurations.Generic.save();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to save configurations!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Saving Friend list");
			Configurations.Friends.save();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to save friend list!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Saving Target list");
			Configurations.Targets.save();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to save target list!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Saving Key Maucros");
			Configurations.KeyMaucros.save();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to save key maucros!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Saving Alts");
			Configurations.Alts.save();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to save alts!");
			Maucros.getLogger().catching(e);
		}
		try {
			Maucros.getLogger().trace("Saving X-Ray block config");
			Configurations.XRay.save();
		} catch (Throwable e) {
			Maucros.getLogger().warning("Failed to save X-Ray block config!");
			Maucros.getLogger().catching(e);
		}
		if (Settings.Enabled.lua) {
			try {
				Maucros.getLogger().trace("Saving Lua Global Data");
				mauluam.MauGDataLib.saveGlobalData();
			} catch (Throwable e) {
				Maucros.getLogger().warning("Failed to save Lua Global Data!");
				Maucros.getLogger().catching(e);
			}
		}
	}
	
	public void setModMetadata(ModMetadata mm) {
		Maucros.getLogger().debug("Changing Mod Metadata...");
		mm.autogenerated = false;
		mm.name = name;
		mm.modId = name;
		mm.version = version;
		mm.authorList = Arrays.asList("Tulir293");
		mm.credits = "Davidee, Darius and the Forge team";
		String installed;
		if (Settings.Enabled.mauevtlib && Settings.Enabled.gui && Settings.Enabled.lua) installed = "Every Recommended Library";
		else installed = (Settings.Enabled.mauevtlib ? "MauEventLib, " : "") + (Settings.Enabled.gui ? "GuiLib, " : "")
				+ (Settings.Enabled.lua ? "LuaJ, " : "") + "Expr";
		mm.description = "A client-only mod that adds a bunch of handy features.\nUses the Gui Library created by Davidee to add the config GUIs, "
				+ "MauEventLib by Tulir293 for MauChat in the Minecraft chat GUI, Expr by Darius (Slightly modified) for the calculator and "
				+ "LuaJ (requires Apache BCEL) for Lua support in Key Maucros.\nInstalled Libraries: " + installed;
		mm.url = "http://maunium.net/forge/maucros.html";
	}
	
	/*
	 * End of config/misc methods. Start of getters.
	 */
	
	/**
	 * Returns the singleton instance of Maucros and creates one if instance has not yet been created.
	 */
	@InstanceFactory
	public static Maucros getInstance() {
		if (instance == null) return new Maucros();
		return instance;
	}
	
	/**
	 * @return The Maucros Logger
	 */
	public static MaucrosLogger getLogger() {
		return MaucrosLogger.getMaucrosLogger();
	}
	
	/**
	 * @return The Chat Logger
	 */
	public static ChatLogger getChatLogger() {
		return ChatLogger.getChatLogger();
	}
	
	/**
	 * @return The singleton instance of {@link #}
	 */
	public Actions getActions() {
		return actions;
	}
	
	/**
	 * @return The singleton instance of {@link MauKeyBinds}
	 */
	public MauKeyBinds getKeybindings() {
		return mauKeyBinds;
	}
	
	/**
	 * @return The singleton instance of {@link KeyInputListener}
	 */
	public KeyInputListener getKeyInputListener() {
		return keyInputListener;
	}
	
	/**
	 * @return The singleton instance of {@link TickListener}
	 */
	public TickListener getTickListener() {
		return tickListener;
	}
	
	/**
	 * @return The singleton instance of {@link OverlayRenderListener}
	 */
	public OverlayRenderListener getOverlayRenderer() {
		return overlayRenderListener;
	}
	
	/**
	 * @return The singleton instance of {@link WorldRenderListener}
	 */
	public WorldRenderListener getWorldRenderer() {
		return worldRenderListener;
	}
	
	/**
	 * @return The singleton instance of {@link InChatListener}
	 */
	public InChatListener getChatInputListner() {
		return inChatListener;
	}
	
	/**
	 * @return The singleton instance of {@link OutChatListener}
	 */
	public OutChatListener getChatOutputListener() {
		if (Settings.Enabled.mauevtlib) return outChatListener;
		else return null;
	}
	
	/**
	 * @return The singleton instance of {@link GuiOpenListener}
	 */
	public GuiOpenListener getGuiOpenListener() {
		return guiOpenListener;
	}
	
	/**
	 * @return The singleton instance of {@link ConnectListener}
	 */
	public ConnectListener getConnectListener() {
		return connectListener;
	}
	
	/**
	 * @return The singleton instance of {@link PacketListener}
	 */
	public PacketListener getPacketListener() {
		return packetListener;
	}
	
	/*
	 * End of getters. End of class.
	 */
}
