package net.maunium.Maucros.Misc;

import net.minecraft.entity.EntityCreature;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.boss.EntityDragon;
import net.minecraft.entity.boss.EntityDragonPart;
import net.minecraft.entity.boss.EntityWither;
import net.minecraft.entity.monster.*;
import net.minecraft.entity.passive.*;
import net.minecraft.entity.player.EntityPlayer;

/**
 * Used to find entity classes by name
 * 
 * @author Tulir293
 * @since 1.4 Alpha 3
 */
public class EntityClassFinder {
	/**
	 * Find an entity class by name
	 * @return The entity class, or null if not found.
	 */
	public static Class<? extends EntityLivingBase> getEntityClass(String s) {
		if (equals(s, "all", "living", "livings", "livingbase", "livingbases")) return EntityLivingBase.class;
		else if (equals(s, "player", "others")) return EntityPlayer.class;
		else if (equals(s, "mob", "mobs", "monsters", "monster")) return EntityMob.class;
		else if (equals(s, "creature", "creatures")) return EntityCreature.class;
		else if (equals(s, "ambientcreature", "ambientcreatures", "ambient", "ambients")) return EntityAmbientCreature.class;
		else if (equals(s, "animal", "animals")) return EntityAnimal.class;
		else if (equals(s, "tameable", "tameables")) return EntityTameable.class;
		else if (equals(s, "water", "watermob", "waters", "watermobs")) return EntityWaterMob.class;
		else if (equals(s, "villager", "villagers")) return EntityVillager.class;
		else {
			for (Class<?> c : passives) {
				try {
					if (c.getSimpleName().substring(6).equalsIgnoreCase(s)) return (Class<? extends EntityLivingBase>) c;
				} catch (Throwable t) {
					continue;
				}
			}
			for (Class<?> c : monsters) {
				try {
					if (c.getSimpleName().substring(6).equalsIgnoreCase(s)) return (Class<? extends EntityLivingBase>) c;
				} catch (Throwable t) {
					continue;
				}
			}
			for (Class<?> c : bosses) {
				try {
					if (c.getSimpleName().substring(6).equalsIgnoreCase(s)) return (Class<? extends EntityLivingBase>) c;
				} catch (Throwable t) {
					continue;
				}
			}
		}
		try {
			return (Class<? extends EntityLivingBase>) Class.forName(s);
		} catch (Exception e) {
			return null;
		}
	}
	
	private static boolean equals(String s, String... ct) {
		for (String c : ct)
			if (s.equalsIgnoreCase(c)) return true;
		return false;
	}
	
	private static Class<?>[] passives = { EntityWolf.class, EntitySquid.class, EntitySheep.class, EntityPig.class, EntityOcelot.class, EntityMooshroom.class,
			EntityHorse.class, EntityCow.class, EntityChicken.class, EntityBat.class };
	private static Class<?>[] monsters = { EntitySpider.class, EntityZombie.class, EntityWitch.class, EntitySnowman.class, EntitySlime.class,
			EntitySkeleton.class, EntitySilverfish.class, EntityPigZombie.class, EntityMagmaCube.class, EntityIronGolem.class, EntityGolem.class,
			EntityGiantZombie.class, EntityGhast.class, EntityEnderman.class, EntityCreeper.class, EntityCaveSpider.class, EntityBlaze.class };
	private static Class<?>[] bosses = { EntityDragon.class, EntityDragonPart.class, EntityWither.class };
	
}
