package net.maunium.Maucros.Misc;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Properties;

import net.maunium.Maucros.Maucros;
import net.maunium.Maucros.Config.Configurations;
import net.maunium.Maucros.Config.Settings;

/**
 * An I18n class for Maucros in order to avoid I18n mixup with other mods.
 * 
 * @author Tulir293
 * @since 1.4 Alpha 3
 */
public class I18n {
	private static Properties lang;
	
	/**
	 * Initializes Maucros I18n using the given InputStream
	 * 
	 * @throws IOException As defined in properties.load(InputStream r)
	 */
	public static void init(InputStream is) throws IOException {
		init(new InputStreamReader(is, "UTF-8"));
	}
	
	/**
	 * Initializes Maucros I18n using the given Reader
	 * 
	 * @throws IOException As defined in properties.load(Reader r)
	 */
	public static void init(Reader r) throws IOException {
		lang = new Properties();
		lang.load(r);
	}
	
	/**
	 * Initializes Maucros I18n using the given language file.
	 * 
	 * @param langDir The directory to look for the language file in.
	 * @param language The language file name to use (without .lang ending)
	 * @throws FileNotFoundException If the given file and the default language file don't exist.
	 * @throws IOException As defined in I18n.init(Reader r)
	 */
	public static void init(File langDir, String language) throws IOException {
		File f = new File(langDir, language + ".lang");
		if (!f.exists()) {
			Maucros.getLogger().warning("Language not found: " + language + ". Using en_US.");
			f = new File(Configurations.getDirectory("lang"), "en_US.lang");
			if (!f.exists()) {
				Maucros.saveLangFiles();
				f = new File(Configurations.getDirectory("lang"), "en_US.lang");
				if (!f.exists()) throw new IOException("Default language file not found.");
			}
		}
		init(new InputStreamReader(new FileInputStream(f), "UTF-8"));
	}
	
	/**
	 * Formats an I18n node with possible arguments.
	 * 
	 * @param node The node to format.
	 * @param replace The arguments to replace {i}'s with.
	 * @return The formatted String.
	 */
	public static String format(String node, Object... replace) {
		if(Settings.disabled) return "Server doesn't allow Maucros";
		if (lang.containsKey(node)) {
			String rtrn = lang.getProperty(node);
			int i = 0;
			for (Object o : replace) {
				rtrn = rtrn.replace("{" + i + "}", o + "");
				i++;
			}
			return rtrn;
		} else return node;
	}
}
