package net.maunium.Maucros.Misc;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.image.BufferedImage;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.nio.IntBuffer;

import javax.imageio.ImageIO;

import org.lwjgl.BufferUtils;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL12;

import net.maunium.Maucros.Maucros;
import net.maunium.Maucros.Config.Settings;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.OpenGlHelper;
import net.minecraft.client.renderer.texture.TextureUtil;

public class ScreenshotTaker {
	public static void capture() {
		if (uploading) {
			Maucros.printChatErrorLocalized("message.screenshot.alreadyuploading");
			return;
		}
		int width, height;
		Minecraft mc = Minecraft.getMinecraft();
		if (OpenGlHelper.isFramebufferEnabled()) {
			width = mc.getFramebuffer().framebufferTextureWidth;
			height = mc.getFramebuffer().framebufferTextureHeight;
		} else {
			width = mc.displayWidth;
			height = mc.displayHeight;
		}
		int totalPixels = width * height;
		IntBuffer pixelBuffer = BufferUtils.createIntBuffer(totalPixels);
		int[] pixelValues = new int[totalPixels];
		GL11.glPixelStorei(GL11.GL_PACK_ALIGNMENT, 1);
		GL11.glPixelStorei(GL11.GL_UNPACK_ALIGNMENT, 1);
		if (OpenGlHelper.isFramebufferEnabled()) {
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, mc.getFramebuffer().framebufferTexture);
			GL11.glGetTexImage(GL11.GL_TEXTURE_2D, 0, GL12.GL_BGRA, GL12.GL_UNSIGNED_INT_8_8_8_8_REV, pixelBuffer);
		} else GL11.glReadPixels(0, 0, width, height, GL12.GL_BGRA, GL12.GL_UNSIGNED_INT_8_8_8_8_REV, pixelBuffer);
		
		pixelBuffer.get(pixelValues);
		TextureUtil.processPixelValues(pixelValues, width, height);
		final File screenshotName = new File(Minecraft.getMinecraft().mcDataDir, "screenshots" + File.separator + "temp.png");
		Maucros.printChatLocalized("message.screenshot.uploading");
		save(pixelValues, width, height, screenshotName);
	}
	
	public static volatile boolean uploading = false;
	
	private static void save(final int[] pixelValues, final int width, final int height, final File saveTo) {
		uploading = true;
		Thread t = new Thread("ScreenshotSaveAndUpload") {
			@Override
			public void run() {
				BufferedImage image;
				if (OpenGlHelper.isFramebufferEnabled()) {
					image = new BufferedImage(width, height, 1);
					double totalPixels = pixelValues.length;
					for (int y = 0; y < height; y++)
						for (int x = 0; x < width; x++)
							image.setRGB(x, y, pixelValues[y * width + x]);
				} else {
					image = new BufferedImage(width, height, 1);
					image.setRGB(0, 0, width, height, pixelValues, 0, width);
				}
				try {
					ImageIO.write(image, "png", saveTo);
					upload(saveTo);
					saveTo.delete();
				} catch (Throwable e) {
					Maucros.getLogger().catching(e);
				}
				uploading = false;
			}
		};
		t.setPriority(5);
		t.start();
	}
	
	private static void upload(File f) throws IOException {
		if (Settings.Screenshot.ip.isEmpty()) {
			Maucros.printChatErrorLocalized("message.screenshot.ip");
			return;
		}
		Socket s = new Socket(InetAddress.getByName(Settings.Screenshot.ip), Settings.Screenshot.port);
		s.setSoTimeout(10000);
		
		long st = System.currentTimeMillis();
		
		DataOutputStream out = new DataOutputStream(s.getOutputStream());
		InputStream in = s.getInputStream();
		
		out.write(0);
		int scr = in.read();
		if (scr != 0) {
			Maucros.printChatErrorLocalized("message.screenshot.response", scr);
			s.close();
			return;
		}
		out.write(3);
		int ctr = in.read();
		if (ctr != 0) {
			Maucros.printChatErrorLocalized("message.screenshot.response", ctr);
			s.close();
			return;
		}
		
		int reqAuth = in.read();
		if (reqAuth == 1) {
			if (Settings.Screenshot.password.isEmpty()) {
				Maucros.printChatErrorLocalized("message.screenshot.password.notfound");
				out.write(0);
				s.close();
				return;
			}
			byte[] pw = Settings.Screenshot.password.getBytes("UTF-8");
			out.write(pw.length);
			out.write(pw);
			int response = in.read();
			if (response != 0) {
				Maucros.printChatErrorLocalized("message.screenshot.password.invalid");
				s.close();
				return;
			}
		}
		
		FileInputStream fis = new FileInputStream(f);
		int x = fis.available();
		// "Uploading - 0% - " + (System.currentTimeMillis() - st) / 1000 + "s"
		out.writeInt(x);
		for (int i = 0; i < x; i++) {
			// "Uploading - " + (int) (i * 100.0f / x) + "% - " + (System.currentTimeMillis() - st) / 1000 + "s"
			out.write(fis.read());
		}
		fis.close();
		
		int response = in.read();
		if (response == 0) {
			int length = in.read();
			byte[] address = new byte[length];
			in.read(address);
			String url = String.format("http://%s/%s.png", Settings.Screenshot.ip, new String(address, "UTF-8"));
			
			Clipboard c = Toolkit.getDefaultToolkit().getSystemClipboard();
			StringSelection ss = new StringSelection(url);
			c.setContents(ss, ss);
			
			out.close();
			Maucros.printChatLocalized("message.screenshot.uploaded", url);
		} else Maucros.printChatErrorLocalized("message.screenshot.response", response);
		
		s.close();
	}
}